VERSION 5.00
Begin VB.Form fDualPivotTest 
   Caption         =   "Sorting-Comparison (Click to Refresh)"
   ClientHeight    =   9015
   ClientLeft      =   120
   ClientTop       =   465
   ClientWidth     =   7695
   BeginProperty Font 
      Name            =   "Arial"
      Size            =   9.75
      Charset         =   0
      Weight          =   400
      Underline       =   0   'False
      Italic          =   0   'False
      Strikethrough   =   0   'False
   EndProperty
   LinkTopic       =   "Form1"
   ScaleHeight     =   9015
   ScaleWidth      =   7695
   StartUpPosition =   3  'Windows-Standard
End
Attribute VB_Name = "fDualPivotTest"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Private Enum enmFillMode
  Random
  EvenOdd
  SawTooths
  PreSortedDesc
  PreSortedAsc
  StairSteps
  Constant
End Enum

Private VerifyResults As Boolean, ArrCount As Long

Private Sub Form_Click()
  AutoRedraw = True: Cls
  
  If App.LogMode Then 'we run compiled
    VerifyResults = False
    ArrCount = 2000000
  Else 'we run in the IDE (so we use Result-Verification and a smaller Array-Count)
    VerifyResults = True
    ArrCount = 200000
  End If
  
  Print vbLf; " Random Data, Array-Size=" & ArrCount
  DoTests Random, VerifyResults

  Print vbLf; " PresortedDesc Data, Array-Size=" & ArrCount
  DoTests PreSortedDesc, VerifyResults

  Print vbLf; " PresortedAsc Data, Array-Size=" & ArrCount
  DoTests PreSortedAsc, VerifyResults

  Print vbLf; " SawTooths Data, Array-Size=" & ArrCount
  DoTests SawTooths, VerifyResults

  Print vbLf; " Even/Odd Data, Array-Size=" & ArrCount
  DoTests EvenOdd, VerifyResults

  Print vbLf; " StairSteps Data, Array-Size=" & ArrCount
  DoTests StairSteps, VerifyResults
  
  Print vbLf; " Constant Data, Array-Size=" & ArrCount
  DoTests Constant, VerifyResults
End Sub

Private Sub DoTests(ByVal FillMode As enmFillMode, Optional ByVal VerifyResults As Boolean)
Dim i As Long, Arr() As Long, ArrVerify() As Long
  If VerifyResults Then FillArray ArrVerify, FillMode: ShellSort ArrVerify, 0, UBound(ArrVerify)

  FillArray Arr, FillMode
  Timing True
    DualPivotQuickSort Arr, 0, UBound(Arr)
  Print , "DualPivot-QuickSort", Timing
  If VerifyResults Then For i = 0 To UBound(Arr): Debug.Assert Arr(i) = ArrVerify(i): Next
  
  FillArray Arr, FillMode
  Timing True
    QuickSortExtended Arr, 0, UBound(Arr)
  Print , "QuickSort Extended", Timing
  If VerifyResults Then For i = 0 To UBound(Arr): Debug.Assert Arr(i) = ArrVerify(i): Next

  FillArray Arr, FillMode
  Timing True
    NaiveQuickSort Arr, 0, UBound(Arr)
  Print , "Naive-QuickSort", Timing
  If VerifyResults Then For i = 0 To UBound(Arr): Debug.Assert Arr(i) = ArrVerify(i): Next
 
'  FillArray Arr, FillMode
'  Timing True
'    HeapSortLongs Arr
'  Print , "Standard-HeapSort", Timing
'  If VerifyResults Then For i = 0 To UBound(Arr): Debug.Assert Arr(i) = ArrVerify(i): Next
'
'  FillArray Arr, FillMode
'  Timing True
'    ShellSort Arr, 0, UBound(Arr)
'  Print , "Standard-ShellSort", Timing
End Sub

Private Sub FillArray(Arr() As Long, ByVal FillMode As enmFillMode)
Dim i As Long, j As Long
  ReDim Arr(0 To ArrCount - 1)
  Rnd -123 'ensure always the same random-sequence for all algos
  Select Case FillMode
    Case Random
      For i = 0 To ArrCount - 1: Arr(i) = Int(Rnd * ArrCount): Next
    Case EvenOdd 'this is extremely inconvenient data for the Naive-Quicksort
      For i = 0 To ArrCount - 1: Arr(i) = i Mod 2: Next
    Case SawTooths
      For i = 0 To ArrCount - 1
        If i Mod 200 = 0 Then j = 0 Else j = j + 1 'increase the next 200 times until we drop again
        Arr(i) = j
      Next
    Case PreSortedDesc
      For i = 0 To ArrCount - 1: Arr(i) = ArrCount - i - 1: Next
    Case PreSortedAsc
      For i = 0 To ArrCount - 1: Arr(i) = i: Next
    Case StairSteps
      For i = 0 To ArrCount - 1
        If i Mod 100 = 0 Then j = Int(Rnd * ArrCount) 'change the const-value for the next 100 array-slots
        Arr(i) = j
      Next
    Case Constant
      For i = 0 To ArrCount - 1: Arr(i) = 1: Next
  End Select

  DoEvents
  CC = 0 'reset the Comparison-Counter before we enter each sort-routine
End Sub

Private Function Timing(Optional ByVal Start As Boolean) As String
Static T!
  If Start Then T = Timer Else Timing = Format$((Timer - T) * 1000, "0msec") & IIf(CC, ", Comparisons=" & Format$(CC / 10 ^ 6, "0.0Mio"), "")
End Function

Private Sub ShellSort(Arr() As Long, ByVal LowIndex&, ByVal HiIndex&) 'used for verification here
Dim i&, j&, Inc&, Tmp As Long
  If LowIndex >= HiIndex Then Exit Sub

  Do: Inc = 3 * Inc + 1: Loop Until Inc > (HiIndex - LowIndex)

  Do
    Inc = (Inc - 1) \ 3
    For i = LowIndex + Inc To HiIndex
      Tmp = Arr(i): j = i - Inc
      Do While j >= LowIndex
        If Arr(j) > Tmp Then Arr(j + Inc) = Arr(j): j = j - Inc Else Exit Do
      Loop
      Arr(j + Inc) = Tmp
    Next i
  Loop While Inc > 1
End Sub
