Attribute VB_Name = "modQuickSort"
Option Explicit

'Indirect String-Sorts

'Port from Java of the Dual-Pivot Quicksort, as suggested by Vladimir Yaroslavskiy in February 2009
'now the default Array-Sorting-algo in the official JDK-sources:
'http://cr.openjdk.java.net/~alanb/6880672/webrev.00/src/share/classes/java/util/Arrays.java.html
Public Sub DualPivotQuickSort(Arr() As String, Ind() As Long, Optional ByVal LB& = -1, Optional ByVal UB& = -1)
Dim i As Long, j As Long, M1 As Long, M2 As Long
Dim P1 As Long, P2 As Long, Tmp As Long  '<- adjust types here, when switching to something different than Long
  
  If LB = -1 And UB = -1 Then 'we just entered the first time, so we init the Indirect-Index-Array once
    LB = LBound(Arr): UB = UBound(Arr): ReDim Ind(LB To UB)
    For i = LB To UB: Ind(i) = i: Next
  End If
  
  If (UB - LB) < 27 Then  ' insertion sort for remaining tiny ranges
    For i = LB To UB - 1
      Tmp = Ind(i + 1)
        For j = i To LB Step -1
          If Arr(Tmp) < Arr(Ind(j)) Then Ind(j + 1) = Ind(j) Else Exit For
        Next j
      Ind(j + 1) = Tmp
    Next i
    Exit Sub
  End If

  M1 = LB + (UB - LB) \ 3: M2 = UB - M1 + LB '"medians"
 
  If Arr(Ind(M1)) < Arr(Ind(M2)) Then i = LB: j = UB Else i = UB: j = LB
  Tmp = Ind(M1): Ind(M1) = Ind(i): Ind(i) = Tmp
  Tmp = Ind(M2): Ind(M2) = Ind(j): Ind(j) = Tmp
 
  P1 = Ind(LB): P2 = Ind(UB) 'pivots
  M1 = LB + 1:  M2 = UB - 1  '... and Medians switched into "pointers"
  
  'sorting
  i = M1 - 1
  Do While i < M2 '<- M2 decreases within this loop, so we cannot use a For Next one
    i = i + 1
 
    If Arr(P1) > Arr(Ind(i)) Then
      Tmp = Ind(i): Ind(i) = Ind(M1): Ind(M1) = Tmp
      M1 = M1 + 1

    ElseIf Arr(P2) < Arr(Ind(i)) Then
    
      Do While M2 > i
        If Arr(P2) < Arr(Ind(M2)) Then M2 = M2 - 1 Else Exit Do
      Loop
      
      If Arr(P1) <= Arr(Ind(M2)) Then
        Tmp = Ind(i): Ind(i) = Ind(M2): Ind(M2) = Tmp
        M2 = M2 - 1
      Else
        Tmp = Ind(i): Ind(i) = Ind(M1): Ind(M1) = Ind(M2): Ind(M2) = Tmp
        M2 = M2 - 1: M1 = M1 + 1
      End If
    End If
  Loop
  
  Tmp = Ind(M1 - 1): Ind(M1 - 1) = Ind(LB): Ind(LB) = Tmp
  Tmp = Ind(M2 + 1): Ind(M2 + 1) = Ind(UB): Ind(UB) = Tmp

  DualPivotQuickSort Arr, Ind, LB, M1 - 2  'recurse into lower subarray
  DualPivotQuickSort Arr, Ind, M2 + 2, UB 'recurse into upper subarray
 
  If (M2 - M1) > (UB - LB - 21) Then  'handle equal elements
    If Arr(P1) <> Arr(P2) Then
      i = M1 - 1
      Do While i < M2 '<- M2 decreases within this loop, so we cannot use a For Next one
        i = i + 1
        If Arr(P1) = Arr(Ind(i)) Then
          Tmp = Ind(i): Ind(i) = Ind(M1): Ind(M1) = Tmp
          M1 = M1 + 1

        ElseIf Arr(P2) = Arr(Ind(i)) Then
          If Arr(P1) <> Arr(Ind(M2)) Then
            Tmp = Ind(i): Ind(i) = Ind(M2): Ind(M2) = Tmp
            M2 = M2 - 1
          Else
            Tmp = Ind(i): Ind(i) = Ind(M1): Ind(M1) = Ind(M2): Ind(M2) = Tmp
            M2 = M2 - 1: M1 = M1 + 1
          End If
        End If
      Loop
    End If
  End If

  If Arr(P1) < Arr(P2) Then DualPivotQuickSort Arr, Ind, M1, M2 'recurse into remaining subarray
End Sub
 
Public Sub QuickSortExtended(Arr() As String, Ind() As Long, Optional ByVal LB& = -1, Optional ByVal UB& = -1)
Dim i As Long, M1 As Long, M2 As Long
Dim Piv As Long, Tmp As Long '<- adjust types here, when switching to something different than Long
  
  If LB = -1 And UB = -1 Then 'we just entered the first time, so we init the Indirect-Index-Array once
    LB = LBound(Arr): UB = UBound(Arr): ReDim Ind(LB To UB)
    For i = LB To UB: Ind(i) = i: Next
  End If
  
  If UB - LB = 1 Then
     M1 = LB
     If Arr(Ind(M1)) > Arr(Ind(UB)) Then Tmp = Ind(M1): Ind(M1) = Ind(UB): Ind(UB) = Tmp
     Exit Sub
  Else
    M1 = (LB + UB) \ 2
    If Arr(Ind(M1)) = Arr(Ind(LB)) Then
      M2 = UB - 1
      M1 = LB
      Do
        M1 = M1 + 1
        If M1 > M2 Then
          If Arr(Ind(UB)) < Arr(Ind(LB)) Then Tmp = Ind(LB): Ind(LB) = Ind(UB): Ind(UB) = Tmp
          Exit Sub
        End If
      Loop Until Arr(Ind(M1)) <> Arr(Ind(LB))
      Piv = Ind(M1)
      If M1 > LB Then
        If Arr(Ind(LB)) > Arr(Piv) Then Ind(M1) = Ind(LB): Ind(LB) = Piv: Piv = Ind(M1)
      End If
    Else
      Piv = Ind(M1)
      M1 = LB
      Do While Arr(Ind(M1)) < Arr(Piv): M1 = M1 + 1: Loop
    End If
  End If
  
  M2 = UB
  Do
    Do While Arr(Ind(M2)) > Arr(Piv): M2 = M2 - 1: Loop
    If M1 <= M2 Then
      Tmp = Ind(M1): Ind(M1) = Ind(M2): Ind(M2) = Tmp 'swap
      M1 = M1 + 1
      M2 = M2 - 1
    End If
    If M1 > M2 Then Exit Do
    Do While Arr(Ind(M1)) < Arr(Piv): M1 = M1 + 1: Loop
  Loop
  If LB < M2 Then QuickSortExtended Arr, Ind, LB, M2
  If M1 < UB Then QuickSortExtended Arr, Ind, M1, UB
End Sub
